' ==============================================
'  Name:    RLE4BAS
'  Type:    Main Module
'  Lang:    Microsoft QuickBASIC Compiler v4.00+
'           with Crescent Software's PDQ.LIB
'  Author:  Mike Welch of Dallas, Texas
'  Comments:PO BOX 401011 : Dallas, Tx 75240
'  Credits: Earl Montgomery and Jerry McBride
'  Date:    9/2/1990
'  Purpose: Using a modified RLE (Run Length
'           Encoding) algorithm, encrypt and
'           compress a standard ASCII file.
'           This file can then be read directly
'           from within QuickBASIC and de-
'           compressed with RLEDecrypt%().
'           Saves about 10-15% on average.
'  Sale:    None.  For public consumption.
' ==============================================
'
DEFINT A-Z					' no floating point

DECLARE FUNCTION RLEDecrypt% (Work$)		' compress/encrypt
DECLARE FUNCTION RLEEncrypt% (Work$)		' decompress/decrypt
DECLARE FUNCTION PDQParse$(C$)			' PDQ-specific func
DECLARE FUNCTION PDQExist%(C$)			' PDQ-specific func

C$ = UCASE$(COMMAND$)				' get command line
CALL SetDelimitChar(32)				' parse with spaces
InFile$ = PDQParse$(C$)				' get infile name
OutFile$ = PDQParse$(C$)			' get outfile name
Switch$ = PDQParse$(C$)				' check for /D

PRINT						' this message is
PRINT "          "		' always printed
PRINT "        "		' anytime you key in
PRINT "           "		' RLE4BAS.
PRINT ""
PRINT "v1.00  (P) 1990 Mike Welch"
PRINT
PRINT "Encrypt/Decrypt/Compress a"
PRINT "standard ASCII file for use"
PRINT "in programming/low security"
PRINT

' Check for paramters, and files on disk
' PDQExist%() only works with Crescent's
' PDQ.LIB (replacement for BCOM).
IF LEN(OutFile$) = 0 THEN			' if bad parameters
	PRINT "RLE4BAS InFile OutFile [/D]"	' print syntax
	END					' exit
ELSEIF PDQExist%(OutFile$) THEN			' don't overwrite
	PRINT OutFile$; " already exists"	' existing file!
	END
ELSEIF PDQExist%(InFile$) = 0 THEN		' infile not found
	PRINT "Cannot find "; OutFile$		' print message
	END					' terminate
END IF

OPEN InFile$ FOR INPUT AS 1			' open input file
OPEN OutFile$ FOR OUTPUT AS 2			' open output file

' Convert files
IF Switch$ = "/D" THEN				' check for decompress
   PRINT "Uncompress/Decrypt "; InFile$;	' print function msg
   DO WHILE NOT EOF(1)				' loop while data in file
      INPUT #1, Work$				' same as LINE INPUT in PDQ
      x% = RLEDecrypt%(Work$)			' x% is a dummy value here
      PRINT #2, Work$				' write decompressed string
   LOOP
   PRINT
   PRINT "Done.  "; OutFile$; " created"	' ending message for /D
ELSE
   PRINT "Compress/Encrypt "; InFile$;		' print compress message
   DO WHILE NOT EOF(1)				' loop while data in file
      INPUT #1, Work$				' read ASCII string
      x% = RLEEncrypt%(Work$)			' attempt to compress
      IF x% < 0 THEN				' if x% = -1, then error
         PRINT
         PRINT "Cannot process "; InFile$	' print terminate message
         END					' exit
      END IF
      PRINT #2, Work$				' write encrypted/compressed
      Saved& = Saved& + x%			' count bytes saved
   LOOP
   PRINT
   PRINT "Saved "; Saved&; "compressing "; OutFile$
END IF	


' ==============================================
'  Name:    RLEEncrypt%
'  Type:    FUNCTION
'  Lang:    Microsoft QuickBASIC v4.00+
'  Author:  Michael Welch
'  Comment: PO BOX 401011 : Dallas, Tx  75240
'  Credits: Earl Montgomery and Jerry McBride
'  Purpose: Encrypt and compress an ASCII
'           standard string.  Savings are about
'           10-15%.  Can be more or less depending
'           upon the amount of repeated chars.
'  Cost:    None.  For public consumption.
'  Syntax:  RetCode% = RLEEncrypt%(Work$)
'  Ret:     RetCode% = -1 indicates error
'           RetCode% = 0 indicates no compression
'           RetCode% = n is the bytes saved
' ==============================================
'
FUNCTION RLEEncrypt%(Work$)

Dm$ = CHR$(2)					' used for delimiter
k% = 0						' clear from last use

StrLen% = LEN(Work$)				' length of work string

IF StrLen% = 0 THEN				' if nothing to do
   RLEEncrypt% = 0				' return code
   EXIT FUNCTION				' and ret to main module
END IF

IF INSTR(Work$, Dm$) THEN			' if there is a CHR$(2)
   RLEEncrypt% = -1				' char in string, then
   EXIT FUNCTION				' it is not standard ASCII
END IF						' and can't be compressed

IF StrLen% = 1 THEN				' special case
   Work$ = CHR$(ASC(Work$) + 1)			' process here
   RLEEncrypt% = 0				' return flag
   EXIT FUNCTION				' return to main
END IF

FOR i% = 1 TO StrLen% - 1			' loop for all chars - 1
   A$ = MID$(Work$, i%, 1)			' get first char
   B$ = MID$(Work$, i% + 1, 1)			' get next char
   C$ = CHR$(ASC(A$) + 1)			' encrypted byte
   IF INSTR(A$, B$) THEN			' if same character
      k% = k% + 1				' INC character counter
   ELSE
      IF k% >= 3 THEN				' enough to compress
         Z$ = Z$ + Dm$ + CHR$(k% + 27) + C$	' encrypt/compress
         k% = 0					' reset repeat count
      ELSEIF k% THEN				' if had some repeats
         Z$ = Z$ + STRING$(k% + 1, C$)		' encrypt and print literal
         k% = 0					' reset counter
      ELSE
         Z$ = Z$ + C$				' just encrypt byte
      END IF
   END IF
NEXT

' Wrap up.  Since the loop goes for the
' LEN(Work$) - 1, there is a character
' left in B$.  Add it if necessary, and
' include other checks.
IF k% >= 3 THEN					' enough to compress
   Z$ = Z$ + Dm$ + CHR$(k% + 27) + C$		' encrypt/compress
ELSEIF k% THEN					' if had SOME repeats
   Z$ = Z$ + STRING$(k% + 1, C$)		' encrypt and add literal
ELSE
   Z$ = Z$ + CHR$(ASC(B$) + 1)			' include last byte
END IF

RLEEncrypt% = StrLen% - LEN(Z$)			' return bytes saved
Work$ = Z$					' return new string
Z$ = ""						' clear work string

END FUNCTION


' ==============================================
'  Name:    RLEDecrypt%
'  Type:    FUNCTION
'  Lang:    Microsoft QuickBASIC v4.00+
'  Author:  Michael Welch
'  Comment: PO BOX 401011 : Dallas, Tx  75240
'  Credits: Earl Montgomery and Jerry McBride
'  Purpose: De-encrypt a string previously
'           compressed and encrypted with the
'           RLEEncrypt%() function.
'  Cost:    None.  For public consumption.
'  Syntax:  RetCode% = RLEDecrypt%(Work$)
'  Ret:     RetCode% = size of uncompressed
'           string (no errors here).
' ==============================================
'
FUNCTION RLEDecrypt%(Work$)

Dm$ = CHR$(2)					' delimiter byte

StrLen% = LEN(Work$)				' get length of string

IF StrLen% = 0 THEN				' check if Work$ = ""
   RLEDecrypt% = 0				' if so, set flag
   EXIT FUNCTION				' ret to main module
END IF

FOR i% = 1 TO StrLen%				' loop for all chars
   A$ = MID$(Work$, i%, 1)			' read next byte
   IF INSTR(A$, Dm$) THEN			' is it a delimiter?
      i% = i% + 1				' increase byte pointer
      LenS% = ASC(MID$(Work$, i%, 1)) - 26	' get len of repeat
      i% = i% + 1				' increase byte pointer
      Char% = ASC(Mid$(Work$, i%, 1)) - 1	' decrypt real byte
      Z$ = Z$ + STRING$(LenS%, Char%)		' add repeating chars
   ELSE						' normal character
      Z$ = Z$ + CHR$(ASC(A$) - 1)		' add decrypted byte
   END IF
NEXT

RLEDecrypt% = LEN(Z$) - StrLen%			' expanded size

Work$ = Z$					' pass back new str
Z$ = ""						' free memory

END FUNCTION
